;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xef88e0658470c9a6ed2e289847355d2e . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x43f03763f497632941448d865c72d66d . "./code.cg2js/cg-printing.fasl")
(#x82864885b5a70a77a93e05d14ffc1f37 . "./code.cg2js/cg-caret.fasl")
(#xf4247f71cff36014086593a0b4e46283 . "./code.cg2js/cg-item-list.fasl")
(#x174419e99eaa6ab02017c1a665281bf5 . "./code.cg2js/cg-progress-indicator.fasl")
(#x77f14166593ca4556dad2ddea52cd514 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xed7c2107fa9343cc1f48709cecc8e0da . "./code.cg2js/cg-outline.fasl")
(#x2164ceb28724412458d84d52dc18032a . "./code.cg2js/cg-palette.fasl")
(#x19037dee755861726eae79d1f0d77ee2 . "./code.cg2js/cg-selected-object.fasl")
(#x157153cd1421645315c09dac0ab04853 . "./code.cg2js/cg-status-bar.fasl")
(#xaabbc47e94dfa43e8871cb21b1e94d15 . "./code.cg2js/cg-wrap-string.fasl")
(#xd0b4d1066734764421335d001a655211 . "./code.cg2js/cg-fill-texture.fasl")
(#x6eac8c9886dc938f5fcd97cd84aee76d . "./code.cg2js/cg-layout.fasl")
(#xb8bea555ebb3bb98acf07bc926f5ae5c . "./code.cg2js/cg-timer.fasl")
(#x3edf02751077a3810a9255cf3e28f1b5 . "./code.cg2js/cg-dialog-item.fasl")
(#x7c70d3982d2f2ba3632f9e9077857a9a . "./code.cg2js/cg-button.fasl")
(#x8b8c1e6456e5bfef7dcb36552151701d . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xf531e006a0f48db1272cc0727618d0fd . "./code.cg2js/cg-tray.fasl")
(#xb177300f75c06710585f37b61f62cd15 . "./code.cg2js/cg-plot-widget.fasl")
(#x229f697973e612030ffa5f9ed2960898 . "./code.cg2js/cg-pixmap.fasl")
(#xfb5cedea32a05130189e91383e47770b . "./code.cg2js/cg-font-dialog.fasl")
(#x25abb18ac32536153c9525221d1d66a5 . "./code.cg2js/cg-os-widget.fasl")
(#x38c3f074b492e3c896c2078f3e17ddce . "./code.cg2js/cg-text-or-combo.fasl")
(#x089005a98b572a9458fc6d5458f70c28 . "./code.cg2js/cg-trackbar.fasl")
(#x42beeb6cf0b06277f7686f09055f0e25 . "./code.cg2js/cg-up-down-control.fasl")
(#x83ca3800aa487ea73724be7dc65d990a . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xa047cda0f813c7ecd636df53ddcb74e2 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x9142819b28567615bddcec2bd04c6977 . "./code.cg2js/cg-intersect-posbox.fasl")
(#xf127992f9f9afe6b25e64e42d95b2ed6 . "./code.cg2js/cg-bitmap-stream.fasl")
(#xf761c0801ab4716998dfe191b933f87c . "./code.cg2js/cg-chart-widget.fasl")
(#xcf8086678f4ee8c1ddfcde6037180541 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xa11d702c1e09843063d2b82e71bdfa4d . "./code.cg2js/cg-group-box.fasl")
(#x093be846690cc7c1c12daf3e01cc07b2 . "./code.cg2js/cg-static-text.fasl")
(#xb84d645674d83f7ac5c7d096e4119446 . "./code.cg2js/cg-class-slot-grid.fasl")
(#xec4b4337f944e7ea0702e101d5f53830 . "./code.cg2js/cg-color-dialog.fasl")
(#xbef58beeac8ec482739916cf9deb3b39 . "./code.cg2js/cg-intersect.fasl")
(#x6fd93a2dc734164a52c6a6fb785a838d . "./code.cg2js/cg-gesture-emulation.fasl")
(#xc2e976c1fd8a18a7493040a3b6e5a34b . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xa759b8ab02a530412ebbceb75b8652ce . "./code.cg2js/cg-html-dialog.fasl")
(#xebee1703ce6e57b7620ed954cb2e0ff0 . "./code.cg2js/cg-lettered-menu.fasl")
(#xf0100702689b98d591ee1c6ca5eed536 . "./code.cg2js/cg-class-grid.fasl")
(#x2876f5dcfb19ab423bc9e740d3589e91 . "./code.cg2js/cg-pixmap-printing.fasl")
(#xe8f285b9fc89ffce375be35676e81066 . "./code.cg2js/cg-calendar.fasl")
(#x5a4e1019301a37b57b685095e8e41860 . "./code.cg2js/cg-editable-text.fasl")
(#x02fe96faff895d75a7b2d4383cd1cad7 . "./code.cg2js/cg-property.fasl")
(#x39f3df04ff6c72101ad7683309d7b1ef . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xb7fec8586b713da869a7b8cee44ff645 . "./code.cg2js/cg-grid-widget.fasl")
(#x44ebf46eb22f66080815e3af9dd9828f . "./code.cg2js/cg-paren-matching.fasl")
(#x55e6765a0c3c57900d00d49472dc5e19 . "./code.cg2js/cg-dev.fasl")
(#x663455b088390afd4f8ce39986d7d261 . "./code.cg2js/cg-drawable.fasl")
(#xa6f91058ce4a91fcf58ef2b1a0d5555f . "./code.cg2js/cg-curve.fasl")
(#xef9de49e2c730a33a9be44ec66e9352e . "./code.cg2js/cg-lisp-widget.fasl")
(#x4ccc1e1713391e479b9b3b2165ba638b . "./code.cg2js/cg-dde-utils.fasl")
(#xd2a70d23d2c5380976d3d5eb6b647e65 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x2a1a09a88cc4db092b0c02af64fed876 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x3c29f0cee83184e5b1464a1875730ccf . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#xd971fb9934ee912540e7d37fc9fcb8dc . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x0d4a8dcc398cfcf7cc7202a22dc6d8c1 . "./code.cg2js/cg-lamp.fasl")
(#x4544cf309e0235b87b11b3d2449685cd . "./code.cg2js/cg-common-control.fasl")
(#x02f5b5218e947f1f953caff92a9ed186 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#xc3487e4dcd107f31c685921ced1ca71d . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x086d498083b2f2ecb5f5c655f74feee1 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#xfcb6a8dc2c5b859a4c65ddd98e648f59 . "./code.cg2js/cg-project-window.fasl")
(#x176f7b929c768fa45799640d9c4e84fe . "./code.cg2js/cg-sample-file-menu.fasl")
(#x1e72de2a269219a20b7275edb1e7bca0 . "./code.cg2js/cg-class-support.fasl")
(#x6e755bb75dd36be700e6fa93f515878e . "./code.cg2js/cg-multi-picture-button.fasl")
(#xedc304da2dfcdb53373d753ae786dcb6 . "./code.cg2js/cg-yes-no-list.fasl")
(#x9bf83657a877034f8319ad50e2205f24 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xff5e9eb81efb7a8765922fcee64009f3 . "./code.cg2js/cg-string-dialog.fasl")
(#xa6341cfc14d0146ba67e4a71567c1205 . "./code.cg2js/cg-menu-tooltip.fasl")
(#xff12a0d499fbfe08461d2bff2fde6e4f . "./code.cg2js/cg-dropping-outline.fasl")
(#x12090600255b484c2afdaca22a0b4e9a . "./code.cg2js/cg.fasl")
(#xf6228b48af4c048fb9b1d9d9ec01cf6d . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x37251664f17c9e3b8fbd115cea34b335 . "./code.cg2js/cg-scaling-stream.fasl")
(#xa95c4cbfa1a0aeb4a8a671cbf4a46bf9 . "./code.cg2js/cg-yes-no-string.fasl")
(#x500b2c86d53e7ce39a4f10e20e587f6b . "./code.cg2js/cg-icon-pixmap.fasl")
(#x9ec14dedc511d23e34eba51948c27e64 . "./code.cg2js/cg-web-browser.fasl")
(#x24a6e0f26f1ca5876f3651c291f18fe6 . "./code.cg2js/cg-clipboard-stack.fasl")
(#xed062a7c73ae690c255a62a27bf5c9a8 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x572f9330209c2871ddc995fc537189d8 . "./code.cg2js/cg-message-dialog.fasl")
(#xd04c571fc920f0753702e4993aa73b0f . "./code.cg2js/cg-comtab.fasl")
(#x47a3e538275ad65d0959b6829a7c93ef . "./code.cg2js/cg-base.fasl")
(#x942ed30895aafd177e0bf37c8019569a . "./code.cg2js/cg-pixmap-widget.fasl")
(#xdfe13b0d2f2e2c08755592499338aefa . "./code.cg2js/cg-text-edit-pane.fasl")
(#x19d7676c9c23ba2c50d4ff001cdb89d5 . "./code.cg2js/cg-utility-dialog.fasl")
(#x92699da8c574278607565414248f0856 . "./code.cg2js/cg-object-editor.fasl")
(#x2b16a86c3191e414d03c5b45a5b72cc4 . "./code.cg2js/cg-edit-in-place.fasl")
(#x025546375bd559715da56a8ea98fc1bb . "./code.cg2js/cg-radio-button.fasl")
(#x5830334f920b7e452585e19835d52c6b . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xa359de43a649003da1f952d3cef489bc . "./code.cg2js/cg-nodes.fasl")
(#x15a7008029f40a1e17d7c7a7dfc9e112 . "./code.cg2js/cg-get-position.fasl")
(#x57d1dcb2d70e9df6bf045dc16a81da50 . "./code.cg2js/cg-directory-dialog.fasl")
(#x28dde8e4010c940d86786953de1c6971 . "./code.cg2js/cg-lisp-text.fasl")
(#x5378335ce0d1275837575734a461dcc3 . "./code.cg2js/cg-combo-box.fasl")
(#xd727ccb53842c4565cc3537b2c750ee0 . "./code.cg2js/cg-text-widget.fasl")
(#x60edbcc1bb49f660adce091317b2011a . "./code.cg2js/cg-list-view.fasl")
(#xeb4f0431c4b011d1a935e04acf33f824 . "./code.cg2js/cg-graphics-context.fasl")
(#xaab01d9f4cfee57924e2c2988300585d . "./code.cg2js/cg-get-pixmap.fasl")
(#xfa84d65ddc1c8eb16961055b1d020ae4 . "./code.cg2js/cg-menu.fasl")
(#xee1284b74c3db1b3a9707773e4cdaf99 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x12a390197034f2fc98f0892e3ed5b61f . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x19db91af78c2890166cda210f5e90476 . "./code.cg2js/cg-tooltip.fasl")
(#x4a1cd05a1fe24eb080290eba04299b05 . "./code.cg2js/cg-html-widget.fasl")
(#x406a5f3550252b53bea92fb51773ec91 . "./code.cg2js/cg-choice-list.fasl")
(#xc4bbed80b0a64a0b69373d472209cd94 . "./code.cg2js/cg-icon.fasl")
(#xf487bd686173fc4598a46342fbe3980a . "./code.cg2js/cg-acache.fasl")
(#x1a38f165b0ecdcfc499ae8abf4614d1e . "./code.cg2js/cg-template-string.fasl")
(#x5ce35d882ad413cf713face45c49be1b . "./code.cg2js/cg-bitmap-pane.fasl")
(#xea762ce630a5ae9a18a75244f50d2b4b . "./code.cg2js/cg-toggling-widget.fasl")
(#xcea1f373bdee48b3bf1eccf5fe94b297 . "./code.cg2js/cg-tab-control.fasl")
(#xd75e5a538e22500a0dac72a61b0feacb . "./code.cg2js/cg-split-bar.fasl")
(#x055e7030eb13fd3674af85e146e5c19d . "./code.cg2js/cg-picture-widget.fasl")
(#x0547b42e48a7b7f0bf27158237b63832 . "./code.cg2js/cg-player.fasl")
(#x522b29603d73007d6a58ed4d2222e42a . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x918a3f0ef794db8061fe2216f4c8adec . "./code.cg2js/cg-choose-printer.fasl")
(#x706257b1889c5be1044f12ac96f47ada . "./code.cg2js/cg-os-window.fasl")
(#x122c438da9f2959f923665c46c11a781 . "./code.cg2js/cg-scroll-bar.fasl")
(#x70c4e19d70038bf9d9cbf426814411f8 . "./code.cg2js/cg-clipboard.fasl")
(#x1418190a8b59c36aa596bf7b91259a1b . "./code.cg2js/cg-find-string-dialog.fasl")
(#x4da4b824c76bcc0a87aebc0e4af3baf6 . "./code.cg2js/cg-hotspot.fasl")
(#xb94710915f8f8fd5543200d38635535a . "./code.cg2js/cg-check-box.fasl")
(#x93fe1559169ca965a5c0d614fc735dd4 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x8a7c5295a606611b574e14a6839583b7 . "./code.cg2js/cg-file-dialog.fasl")
(#x6d90e1518ec0fb1598e107f41d067cde . "./code.cg2js/cg-toolbar.fasl")
(#x664a8f29037f8567395de8aae77511c9 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xec3fccdc18504c34a3132d1923bf4694 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x9dffec54b9b4b365d4993f612e0373e2 . "./code.cg2js/cg-shortcut-menu.fasl")
(#xfce00ede104e98b4c966e1ed4b3f3edf . "./code.idejs/ide-project.fasl")
(#xf56e99313163942d3a38d96f24735812 . "./code.idejs/ide-intro.fasl")
(#x161594299de485e374a726eec40e0147 . "./code.idejs/ide-find-in-files.fasl")
(#xf0fc2d0a834a8932450d6f2c49628abd . "./code.idejs/ide-grapher.fasl")
(#xf79d45e839700de1f35b3668ceaa5240 . "./code.idejs/ide-inspector.fasl")
(#x446d129444a23989220acf200de942c7 . "./code.idejs/ide-editor.fasl")
(#x6d06501af7ecdeb488ef12c4b0db422a . "./code.idejs/ide-window-list.fasl")
(#x59418a9f85602d95afd968762b1d7433 . "./code.idejs/ide-process-browser.fasl")
(#x3cfbbd83383409e7ab074b61a2cd8c5b . "./code.idejs/ide-patch-dialog.fasl")
(#xf8bca14d1dda36f8ee5176eb0becb5f6 . "./code.idejs/ide-class-grapher.fasl")
(#xf4bb6d9354915264728fe0a43b8923d7 . "./code.idejs/ide-base.fasl")
(#x479d2e7ca3f90a7d97c9527841af1237 . "./code.idejs/ide.fasl")
(#x032069bc2613ab30c2f0fa5afd20c430 . "./code.idejs/ide-trace.fasl")
(#x2f4bbaf965f89c2957421d479ccfde5c . "./code.idejs/ide-form.fasl")
(#x7582f0a6ebe89ffb4b9729eccf6562ea . "./code.idejs/ide-find-definitions.fasl")
(#xb8882a43dfb035aa859e990d84bc8f30 . "./code.idejs/ide-apropos.fasl")
(#x371bf75ca00a4d51b6a39b8afb9e5c80 . "./code.idejs/ide-class-browser.fasl")
(#x6a58ad8058b375bb4f61195a475a7110 . "./code.idejs/ide-navigator.fasl")
(#x7b7fa13e1592816d6e211df1651360e1 . "./code.idejs/ide-clipboard.fasl")
(#xaf231ceec8652aed0cdbf6e3cb37cf20 . "./code.idejs/ide-acache.fasl")
(#xdf950f995bfb40313da2f58ebe9e9532 . "./code.idejs/ide-keyshort.fasl")
(#xa9b939f796d60a84f3af0a8556c23c24 . "./code.idejs/ide-package-list.fasl")
(#x89a5f51c8245784f2ff7bd607e7f0391 . "./code.idejs/ide-dev.fasl")
(#x4469068b47e063cb9811630bd05af4c2 . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
